﻿using System;

namespace UMK.WzorceStrukturalne
{
    public interface IUrządzenieRTV
    {
        void Włącz();
        void Wyłącz();
        void UstawKanał(int numerKanału);
    }

    public class Telewizor : IUrządzenieRTV
    {
        public void Włącz()
        {
            Console.WriteLine("Telewizor zostal wlaczony");
        }
        public void Wyłącz()
        {
            Console.WriteLine("Telewizor zostal wylaczony");
        }
        public void UstawKanał(int numerKanału)
        {
            Console.WriteLine("Kanal w telewizorze zmieniony na: " + numerKanału);
        }
    }

    public class Radio : IUrządzenieRTV
    {
        public void Włącz()
        {
            Console.WriteLine("Radio zostalo wlaczone");
        }
        public void Wyłącz()
        {
            Console.WriteLine("Radio zostalo wylaczone");
        }
        public void UstawKanał(int numerKanału)
        {
            Console.WriteLine("Kanal w radiu zmieniony na: " + numerKanału);
        }
    }

    class Pilot
    {
        public enum RodzajUrządzenia { Domyślne, Telewizor, Radio };

        int bieżącyKanał = -1, poprzedniKanał = -1;

        IUrządzenieRTV urządzenieRTV;

        public Pilot(RodzajUrządzenia rodzajUrządzenia = RodzajUrządzenia.Domyślne)
        {
            switch(rodzajUrządzenia)
            {
                default:
                case RodzajUrządzenia.Telewizor:
                    urządzenieRTV = new Telewizor();
                    break;
                case RodzajUrządzenia.Radio:
                    urządzenieRTV = new Radio();
                    break;
            }
        }

        public void Włącz()
        {
            urządzenieRTV.Włącz();
            UstawKanał(1);
        }

        public void Wyłącz()
        {
            urządzenieRTV.Wyłącz();
        }

        public void UstawKanał(int nowyKanał)
        {
            poprzedniKanał = bieżącyKanał;
            bieżącyKanał = nowyKanał;
            urządzenieRTV.UstawKanał(nowyKanał);
        }

        public void PrzywróćKanał()
        {
            UstawKanał(poprzedniKanał);
        }
    }

    /*
    class PilotZespolony
    {
        int bieżącyKanał = -1, poprzedniKanał = -1;

        IUrządzenieRTV[] urządzeniaRTV;

        public PilotZespolony(params IUrządzenieRTV[] urządzeniaRTV)
        {
            this.urządzeniaRTV = urządzeniaRTV;
        }

        public void Włącz()
        {
            foreach(IUrządzenieRTV urządzenieRTV in urządzeniaRTV)
                urządzenieRTV.Włącz();
            UstawKanał(1);
        }

        public void Wyłącz()
        {
            foreach (IUrządzenieRTV urządzenieRTV in urządzeniaRTV)
                urządzenieRTV.Wyłącz();
        }

        public void UstawKanał(int nowyKanał)
        {
            poprzedniKanał = bieżącyKanał;
            bieżącyKanał = nowyKanał;
            foreach (IUrządzenieRTV urządzenieRTV in urządzeniaRTV)
                urządzenieRTV.UstawKanał(nowyKanał);
        }

        public void PrzywróćKanał()
        {
            UstawKanał(poprzedniKanał);
        }
    }
    */
}
